package com.unleashedsoftware.sdk.network;

import com.google.inject.*;
import java.io.*;

public class RestClient {
    private RestClientResponseReader responseReader;
    private RestClientRequestWriter requestWriter;
    private RestClientConnectionConfigurationCreator connectionConfigurationCreator;

    @Inject
    public RestClient(RestClientResponseReader responseReader,
                      RestClientRequestWriter requestWriter,
                      RestClientConnectionConfigurationCreator connectionConfigurationCreator) {
        this.responseReader = responseReader;
        this.requestWriter = requestWriter;
        this.connectionConfigurationCreator = connectionConfigurationCreator;
    }

    public void get(RestClientReadOnlyRequest request) throws IOException {
        RestClientReadOnlyConnection connection = null;

        try {
            RestClientConnectionConfiguration configuration = connectionConfigurationCreator.create(RestClientRequestMethod.get, request);
            connection = RestClientReadOnlyConnection.create(configuration);
            connection.connect();
            request.requestResult = responseReader.read(connection);
        } catch (Exception exception) {
            log(exception);
        } finally {
            request.requestResultHttpCode = connection.getResponseCode();
            connection.disconnect();
        }
    }

    public void post(RestClientReadWriteRequest request) throws IOException {
        RestClientReadWriteConnection connection = null;

        try {
            RestClientConnectionConfiguration configuration = connectionConfigurationCreator.create(RestClientRequestMethod.post, request);
            connection = RestClientReadWriteConnection.create(configuration);
            connection.connect();
            requestWriter.write(request.requestBody, connection);
            request.requestResult = responseReader.read(connection);
        } catch (Exception exception) {
            log(exception);
        } finally {
            request.requestResultHttpCode = connection.getResponseCode();
            connection.disconnect();
        }
    }

    public void put(RestClientReadWriteRequest request) throws IOException {
        RestClientReadWriteConnection connection = null;

        try {
            RestClientConnectionConfiguration configuration = connectionConfigurationCreator.create(RestClientRequestMethod.put, request);
            connection = RestClientReadWriteConnection.create(configuration);
            connection.connect();
            requestWriter.write(request.requestBody, connection);
            request.requestResult = responseReader.read(connection);
        } catch (Exception exception) {
            log(exception);
        } finally {
            request.requestResultHttpCode = connection.getResponseCode();
            connection.disconnect();
        }
    }

    public void delete(RestClientReadOnlyRequest request) throws IOException {
        RestClientReadOnlyConnection connection = null;

        try {
            RestClientConnectionConfiguration configuration = connectionConfigurationCreator.create(RestClientRequestMethod.delete, request);
            connection = RestClientReadOnlyConnection.create(configuration);
            connection.connect();
            request.requestResult = responseReader.read(connection);
        } catch (Exception exception) {
            log(exception);
        } finally {
            request.requestResultHttpCode = connection.getResponseCode();
            connection.disconnect();
        }
    }

    private void log(Exception exception) {
        throw new LogNotSupportedException();
    }

    public static class LogNotSupportedException extends RuntimeException {
    }
}