package com.unleashedsoftware.sdk.network;

import java.io.*;

public class RestClientReadOnlyConnection extends RestClientConnection {
    private RestClientReadOnlyConnection(RestClientConnectionConfiguration configuration) throws IOException {
        super(configuration);
    }

    public static RestClientReadOnlyConnection create(RestClientConnectionConfiguration configuration) throws IOException {
        if(configuration.requestMethod == RestClientRequestMethod.post)
            throw new PostRequestMethodNotSupportedException();
        if(configuration.requestMethod == RestClientRequestMethod.put)
            throw new PutRequestMethodNotSupportedException();
        return new RestClientReadOnlyConnection(configuration);
    }

    protected boolean isReadable() {
        return true;
    }

    protected boolean isWritable() {
        return false;
    }

    public static class PostRequestMethodNotSupportedException extends RuntimeException {
    }

    public static class PutRequestMethodNotSupportedException extends RuntimeException {
    }
}
