package com.unleashedsoftware.sdk.security;

import org.apache.commons.codec.binary.Base64;
import javax.crypto.*;
import javax.crypto.spec.*;
import java.security.*;

public class UnleashedApiAuthSignatureCalculator {
    private final static String charsetName = "UTF-8";
    private final static String algorithmName = "HmacSHA256";

    public void calculate(UnleashedApiCredentials apiCredentials) throws Exception {
        byte[] hash = apiCredentials.hash.getBytes(charsetName);
        byte[] privateKey = apiCredentials.privateKey.getBytes(charsetName);
        apiCredentials.authSignature = getSignature(hash, privateKey);
    }

    private String getSignature(byte[] hash, byte[] privateKey) throws NoSuchAlgorithmException, InvalidKeyException {
        byte[] signature;

        Mac mac = Mac.getInstance(algorithmName);
        mac.init(new SecretKeySpec(privateKey, algorithmName));
        signature = mac.doFinal(hash);
        mac.reset();

        return Base64.encodeBase64String(signature);
    }
}
